==> 2.0.4.tar.gz: ios/Classes/Plugins/Navigator/NavigatorPlugin.swift <==
//
//  NavigatorPlugin.swift
//  fw_flutter_sdk
//
//  Created by Jeff Zheng on 2022/4/18.
//

import FireworkVideo
import Flutter
import Foundation
import UIKit

private struct PlayerExitButtonInfo {
    enum ExitButtonType: Int {
        case close, floating
    }
    var button: UIButton
    var type: ExitButtonType
}

class NavigatorPlugin: NSObject, FlutterPlugin {
    var channel: FlutterMethodChannel! = nil

    static func register(with registrar: FlutterPluginRegistrar) {
        let channel = FlutterMethodChannel(
            name: ChannelName.fwNavigatorModule.rawValue,
            binaryMessenger: registrar.messenger())
        let instance = NavigatorPlugin()
        instance.channel = channel
        registrar.addMethodCallDelegate(instance, channel: channel)
    }

    func handle(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
        switch call.method {
        case NavigatorPluginMethodName.popNativeContainer.rawValue:
            popNativeContainer(result: result)
        case NavigatorPluginMethodName.startFloatingPlayer.rawValue:
            startFloatingPlayer(result: result)
        case NavigatorPluginMethodName.stopFloatingPlayer.rawValue:
            stopFloatingPlayer(result: result)
        default:
            result(FlutterMethodNotImplemented)
        }
    }

    func popNativeContainer(result: @escaping FlutterResult) {
        if let presentedVC = UIViewController.presentedViewController(),
           Self.isIOSSDKViewController(presentedVC),
           let presentingVC = presentedVC.presentingViewController,
           presentingVC.presentingViewController == UIViewController.keyWindow()?.rootViewController,
           Self.isIOSSDKViewController(presentingVC) {
            presentedVC.dismiss(animated: false) {
                DispatchQueue.main.asyncAfter(wallDeadline: .now() + 0.1) {
                    if let buttonInfo = Self.getPlayerExitButtonInfo(view: presentingVC.view) {
                        if buttonInfo.type == .close {
                            buttonInfo.button.sendActions(for: .touchUpInside)
                            result(true)
                        } else {
                            result(false)
                        }
                    } else {
                        presentingVC.dismiss(animated: true) {
                            result(true)
                        }
                    }
                }
            }
        } else if let rootVC = UIViewController.keyWindow()?.rootViewController,
                    let presentedVC = rootVC.presentedViewController,
                  Self.isIOSSDKViewController(presentedVC) {
            if let buttonInfo = Self.getPlayerExitButtonInfo(view: presentedVC.view) {
                if buttonInfo.type == .close {
                    buttonInfo.button.sendActions(for: .touchUpInside)
                    result(true)
                } else {
                    result(false)
                }
            } else {
                presentedVC.dismiss(animated: true) {
                    result(true)
                }
            }
        } else {
            result(false)
        }
    }

    func startFloatingPlayer(result: @escaping FlutterResult) {
        do {
            try PictureInPictureController.start()
            result(true)
        } catch {
            result(false)
        }
    }

    func stopFloatingPlayer(result: @escaping FlutterResult) {
        do {
            try PictureInPictureController.stop()
            result(true)
        } catch {
            result(false)
        }
    }
}

extension NavigatorPlugin {
    private static func isTargetIOSSDKImage(_ image: UIImage, targetImageName: String) -> Bool {
        let iOSSDKBundle = Bundle(for: FireworkVideoSDK.self)
        let targetImage = UIImage(named: targetImageName, in: iOSSDKBundle, compatibleWith: nil)

        if image.isEqual(targetImage) {
            return true
        }

        if let cgImage = image.cgImage,
           let targetCgImage = targetImage?.cgImage,
           cgImage == targetCgImage {
            return true
        }

        if let ciImage = image.ciImage,
           let targetCiImage = targetImage?.ciImage,
           ciImage == targetCiImage {
            return true
        }

        return false
    }

    private static func getPlayerExitButtonInfo(view: UIView) -> PlayerExitButtonInfo? {
        if let button = view as? UIButton, let image = button.image(for: .normal) {
            if isTargetIOSSDKImage(image, targetImageName: "closeX") {
                return PlayerExitButtonInfo(button: button, type: .close)
            }

            if isTargetIOSSDKImage(image, targetImageName: "down-arrow-light") {
                return PlayerExitButtonInfo(button: button, type: .floating)
            }
        }

        for subview in view.subviews {
            if let result = getPlayerExitButtonInfo(view: subview) {
                return result
            }
        }

        return nil
    }

    private static func isIOSSDKViewController(_ controller: UIViewController?) -> Bool {
        guard let controller = controller else {
            return false
        }

        let iOSSDKBundle = Bundle(for: FireworkVideoSDK.self)
        if Bundle(for: type(of: controller)) == iOSSDKBundle {
            return true
        }

        return isIOSSDKViewController(controller.parent)
    }
}

==> 2.4.6.tar.gz: ios/Classes/Plugins/Navigator/NavigatorPlugin.swift <==
//
//  NavigatorPlugin.swift
//  fw_flutter_sdk
//
//  Created by Jeff Zheng on 2022/4/18.
//

import FireworkVideo
import Flutter
import Foundation
import UIKit

private struct PlayerExitButtonInfo {
    enum ExitButtonType: Int {
        case close, floating
    }
    var button: UIButton
    var type: ExitButtonType
}

class NavigatorPlugin: NSObject, FlutterPlugin {
    private static var hasRegistered = false
    static let shared = NavigatorPlugin()
    
    var channel: FlutterMethodChannel? = nil

    static func register(with registrar: FlutterPluginRegistrar) {
        let channel = FlutterMethodChannel(
            name: ChannelName.fwNavigatorModule.rawValue,
            binaryMessenger: registrar.messenger())
        let instance = hasRegistered ? NavigatorPlugin() : shared
        instance.channel = channel
        registrar.addMethodCallDelegate(instance, channel: channel)
        hasRegistered = true
    }

    func handle(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
        switch call.method {
        case NavigatorPluginMethodName.popNativeContainer.rawValue:
            popNativeContainer(result: result)
        case NavigatorPluginMethodName.startFloatingPlayer.rawValue:
            startFloatingPlayer(result: result)
        case NavigatorPluginMethodName.stopFloatingPlayer.rawValue:
            stopFloatingPlayer(result: result)
        default:
            result(FlutterMethodNotImplemented)
        }
    }

    func popNativeContainer(result: @escaping FlutterResult) {
        if let presentedVC = UIViewController.presentedViewController(),
           Self.isIOSSDKViewController(presentedVC),
           let presentingVC = presentedVC.presentingViewController,
           presentingVC.presentingViewController == UIViewController.keyWindow()?.rootViewController,
           Self.isIOSSDKViewController(presentingVC) {
            presentedVC.dismiss(animated: false) {
                DispatchQueue.main.asyncAfter(wallDeadline: .now() + 0.1) {
                    if let buttonInfo = Self.getPlayerExitButtonInfo(view: presentingVC.view) {
                        if buttonInfo.type == .close {
                            buttonInfo.button.sendActions(for: .touchUpInside)
                            result(true)
                        } else {
                            result(false)
                        }
                    } else {
                        presentingVC.dismiss(animated: true) {
                            result(true)
                        }
                    }
                }
            }
        } else if let rootVC = UIViewController.keyWindow()?.rootViewController,
                    let presentedVC = rootVC.presentedViewController,
                  Self.isIOSSDKViewController(presentedVC) {
            if let buttonInfo = Self.getPlayerExitButtonInfo(view: presentedVC.view) {
                if buttonInfo.type == .close {
                    buttonInfo.button.sendActions(for: .touchUpInside)
                    result(true)
                } else {
                    result(false)
                }
            } else {
                presentedVC.dismiss(animated: true) {
                    result(true)
                }
            }
        } else {
            result(false)
        }
    }

    func startFloatingPlayer(result: @escaping FlutterResult) {
        do {
            try PictureInPictureController.start()
            result(true)
        } catch {
            result(false)
        }
    }

    func stopFloatingPlayer(result: @escaping FlutterResult) {
        do {
            try PictureInPictureController.stop()
            result(true)
        } catch {
            result(false)
        }
    }
}

extension NavigatorPlugin {
    private static func isTargetIOSSDKImage(_ image: UIImage, targetImageName: String) -> Bool {
        let iOSSDKBundle = Bundle(for: FireworkVideoSDK.self)
        let targetImage = UIImage(named: targetImageName, in: iOSSDKBundle, compatibleWith: nil)

        if image.isEqual(targetImage) {
            return true
        }

        if let cgImage = image.cgImage,
           let targetCgImage = targetImage?.cgImage,
           cgImage == targetCgImage {
            return true
        }

        if let ciImage = image.ciImage,
           let targetCiImage = targetImage?.ciImage,
           ciImage == targetCiImage {
            return true
        }

        return false
    }

    private static func getPlayerExitButtonInfo(view: UIView) -> PlayerExitButtonInfo? {
        if let button = view as? UIButton, let image = button.image(for: .normal) {
            if isTargetIOSSDKImage(image, targetImageName: "closeX") {
                return PlayerExitButtonInfo(button: button, type: .close)
            }

            if isTargetIOSSDKImage(image, targetImageName: "down-arrow-light") {
                return PlayerExitButtonInfo(button: button, type: .floating)
            }
        }

        for subview in view.subviews {
            if let result = getPlayerExitButtonInfo(view: subview) {
                return result
            }
        }

        return nil
    }

    private static func isIOSSDKViewController(_ controller: UIViewController?) -> Bool {
        guard let controller = controller else {
            return false
        }

        let iOSSDKBundle = Bundle(for: FireworkVideoSDK.self)
        if Bundle(for: type(of: controller)) == iOSSDKBundle {
            return true
        }

        return isIOSSDKViewController(controller.parent)
    }
}
